@@ -3,6 +3,7 @@ from __future__ import annotations
 import os
 import pkgutil
 from io import StringIO
+from typing import List
 
 import snowflake.connector
 from snowflake.connector.cursor import SnowflakeCursor
@@ -293,6 +294,33 @@ class SnowflakeConnector:
             },
         )
 
+    def setProcedureComment(
+        self,
+        database,
+        schema,
+        role,
+        warehouse,
+        signature=None,
+        name=None,
+        inputParameters=None,
+        show_exceptions=True,
+        comment="",
+    ) -> SnowflakeCursor:
+        if signature is None and name and inputParameters:
+            signature = name + self.generate_signature_from_params(inputParameters)
+        return self.runSql(
+            "set_procedure_comment",
+            {
+                "database": database,
+                "schema": schema,
+                "role": role,
+                "warehouse": warehouse,
+                "signature": signature,
+                "comment": comment,
+            },
+            show_exceptions,
+        )
+
     def putStage(
         self,
         database,
@@ -318,6 +346,21 @@ class SnowflakeConnector:
             },
         )
 
+    def removeFromStage(
+        self, database, schema, role, warehouse, name, path
+    ) -> SnowflakeCursor:
+        return self.runSql(
+            "remove_from_stage",
+            {
+                "database": database,
+                "schema": schema,
+                "role": role,
+                "warehouse": warehouse,
+                "name": name,
+                "path": path,
+            },
+        )
+
     def createStage(
         self,
         database,
@@ -500,7 +543,7 @@ class SnowflakeConnector:
         drop_stage: bool = True,
     ) -> SnowflakeCursor:
 
-        drop_command = f"drop stage {name}_stage;" if drop_stage else ""
+        drop_command = f'drop stage "{name}_STAGE";' if drop_stage else ""
 
         return self.runSql(
             "drop_streamlit",
@@ -600,4 +643,6 @@ class SnowflakeConnector:
             raise (e)
 
     def generate_signature_from_params(self, params: str) -> str:
-        return "(" + " ".join(params.strip("()").split()[1::2]) + ")"
+        if params == "()":
+            return "()"
+        return "(" + " ".join(params.split()[1::2])
