@@ -8,12 +8,14 @@ import re
 import shutil
 import warnings
 import zipfile
+from pathlib import Path
 from typing import Literal
 
 import click
 import requests
 import requirements
 import typer
+from jinja2 import Environment, FileSystemLoader
 from rich import box, print
 from rich.table import Table
 from snowflake.connector.cursor import SnowflakeCursor
@@ -53,7 +55,9 @@ def getDeployNames(database, schema, name) -> dict:
 
 
 def prepareAppZip(file_path, temp_dir) -> str:
-    temp_path = temp_dir + "/app.zip"
+    # get filename from file path (e.g. app.zip from /path/to/app.zip)
+    file_name = os.path.basename(file_path)
+    temp_path = temp_dir + "/" + file_name
     shutil.copy(file_path, temp_path)
     return temp_path
 
@@ -179,12 +183,66 @@ def getPackageNameFromMetadata(metadata_file_path: str) -> str | None:
         return results.group(1)
 
 
+def generateSnowparkCoverageWrapper(
+    target_file: str,
+    proc_name: str,
+    proc_signature: str,
+    handler_module: str,
+    handler_function: str,
+    coverage_reports_stage: str,
+    coverage_reports_stage_path: str,
+):
+    """Using a hardcoded template (python_templates/snowpark_coverage.py.jinja), substitutes variables
+    and writes out a file.
+    The resulting file can be used as the initial handler for the stored proc, and uses the coverage package
+    to measure code coverage of the actual stored proc code.
+    Afterwards, the handler persists the coverage report to json by executing a query.
+
+    Args:
+        target_file (str): _description_
+        proc_name (str): _description_
+        proc_signature (str): _description_
+        handler_module (str): _description_
+        handler_function (str): _description_
+    """
+    templates_path = os.path.join(Path(__file__).parent, "python_templates")
+    environment = Environment(loader=FileSystemLoader(templates_path))
+    template = environment.get_template("snowpark_coverage.py.jinja")
+    content = template.render(
+        {
+            "proc_name": proc_name,
+            "proc_signature": proc_signature,
+            "handler_module": handler_module,
+            "handler_function": handler_function,
+            "coverage_reports_stage": coverage_reports_stage,
+            "coverage_reports_stage_path": coverage_reports_stage_path,
+        }
+    )
+    with open(target_file, "w", encoding="utf-8") as output_file:
+        output_file.write(content)
+
+
+def addFileToExistingZip(zip_file: str, other_file: str):
+    """Adds another file to an existing zip file
+
+    Args:
+        zip_file (str): The existing zip file
+        other_file (str): The new file to add
+    """
+    with zipfile.ZipFile(zip_file, mode="a") as myzip:
+        myzip.write(other_file, os.path.basename(other_file))
+
+
 def installPackages(
-    file_name: str,
+    file_name: str | None,
     perform_anaconda_check: bool = True,
     package_native_libraries: YesNoAskOptionsType = "ask",
+    package_name: str | None = None,
 ) -> tuple[bool, dict[str, list[str]] | None]:
-    os.system(f"pip install -t .packages/ -r {file_name}")
+    if file_name is not None:
+        os.system(f"pip install -t .packages/ -r {file_name}")
+    if package_name is not None:
+        os.system(f"pip install -t .packages/ {package_name}")
     second_chance_results = None
     if perform_anaconda_check:
         click.echo("Checking for dependencies available in Anaconda...")
@@ -243,7 +301,9 @@ def installPackages(
             shutil.rmtree(".packages")
             return False, second_chance_results
     else:
-        click.echo("No native libraries found in packages (Good news!)...")
+        click.echo(
+            "No non-supported native libraries found in packages (Good news!)..."
+        )
         return True, second_chance_results
 
 
