@@ -6,14 +6,15 @@ from snowflake.snowpark import Session
 
 
 def hello(session: Session) -> str:
-    return 'Hello World!'
+    return "Hello World!"
 
 
 # For local debugging. Be aware you may need to type-convert arguments if
 # you add input parameters
-if __name__ == '__main__':
+if __name__ == "__main__":
     from local_connection import get_dev_config
-    session = Session.builder.configs(get_dev_config('dev')).create()
+
+    session = Session.builder.configs(get_dev_config("dev")).create()
     if len(sys.argv) > 1:
         print(hello(session, *sys.argv[1:]))  # type: ignore
     else:
